<?php

/**
 * @file
 * Affirm administration function.
 */

/**
 * Page callback for Affirm redirect.
 */
function uc_affirm_cancel() {
  unset($_SESSION['cart_order']);
  drupal_set_message(t('Your Affirm payment was cancelled. Please feel free to continue shopping or contact us for assistance.'));
  drupal_goto('cart');
}

/**
 * Page callback for Affirm authentication.
 */
function uc_affirm_authentication() {
  if ($_SERVER["REQUEST_METHOD"] == "POST") {
    $checkout_token = $_POST["checkout_token"];
  }
  watchdog('Checkout token', "!message<br/>Checkout Token: !txn_type<br/><br/>", array(
    '!message' => 'Checkout token againt the Affirm authentication',
    '!txn_type' => $checkout_token,
  ), WATCHDOG_WARNING);
  // Exit now if the $_POST was empty.
  if (empty($checkout_token)) {
    watchdog('uc_affirm', 'Affirm response returned without POST parameters.', array(), WATCHDOG_WARNING);
    drupal_set_message(t('Affirm authentication failed!'), 'error');
    drupal_goto('cart');
    return FALSE;
  }

  $data = array(
    'checkout_token' => $checkout_token,
  );
  $order = array();
  return uc_affirm_process_transaction($order, $data);
}

/**
 * Page callback for Affirm void.
 *
 * @param string $charge_id
 *   The payment transaction returned id.
 */
function uc_affirm_void($charge_id = NULL) {
  $response = uc_affirm_api_request(UC_AFFIRM_VOID, $order = array(), $charge_id, $data = array());
  if (!empty($response)) {
    db_update('uc_affirm')
        ->fields(array(
          'status' => 'void',
        ))
        ->condition('charge_id', $charge_id, '=')
        ->execute();
  }
  return $response;
}

/**
 * Page callback to handle a complete Affirm Payment sale.
 *
 * @param int $order_id
 *   Unique id for a order.
 */
function uc_affirm_complete($order_id = 0) {
  if (intval($_SESSION['cart_order']) != $order_id) {
    $_SESSION['cart_order'] = $order_id;
  }

  if (FALSE === uc_order_load($order_id, FALSE)) {
    drupal_goto('cart');
  }
  // This lets us know it's a legitimate access of the complete page.
  $_SESSION['uc_checkout'][$_SESSION['cart_order']]['do_complete'] = TRUE;
  drupal_goto('cart/checkout/complete');
}

/**
 * Process the payment transaction with the info received from Affirm.
 *
 * @param object $order
 *   The loaded order that is being processed.
 * @param array $data
 *   Data to send to Affirm.
 *
 * @return bool
 *   Returns TRUE if the transaction was successful. FALSE if it was not.
 */
function uc_affirm_process_transaction($order, $data) {
  // Query to API to ask for a Authorization payment.
  $response = uc_affirm_api_request(UC_AFFIRM_AUTH_ONLY, $order, array(), $data);
  $order_id = intval($_SESSION['cart_order']);
  $order = uc_order_load($order_id, FALSE);
  if (empty($response)) {
    drupal_set_message(t('We could not complete your payment with Affirm. Please try again or contact us if the problem persists.'), 'error');
    return FALSE;
  }
  if (empty($order_id)) {
    drupal_set_message(t('Your cart is emplty. please feel free to continue your purchasing!'), 'error');
    drupal_goto('cart');
  }
  /*
   * In case the payment method is configured for Auth & Capture, the capture
   * request is triggered right after the authorization.
   */
  if (isset($response['status']) == 'authorized') {
    uc_order_comment_save($order_id, 0, t('Payment order Initialized through Affirm. Order amount : @amount @currency', array('@amount' => $order->order_total, '@currency' => 'USD')), 'order', 'order_under_review');
    // Logging transaction details.
    uc_affirm_transaction_save($order_id, $response['id'], $order->order_total, $response['status'], $order->primary_email, $response['id']);
    if (UC_AFFIRM_AUTH_CAPTURE == variable_get('uc_affirm_txn_type')) {

      $response = uc_affirm_api_request(UC_AFFIRM_CAPTURE_ONLY, $order, $response['id'], $data);
      if (empty($response)) {
        drupal_set_message(t('We could not complete your payment with Affirm. Please try again or contact us if the problem persists.'), 'error');
        return FALSE;
      }
      else {
        if (isset($response['code']) && $response['code'] == 'capture-declined') {
          drupal_set_message(t('Affirm amount capture failed, Please try agin !!'), 'error');
          drupal_goto('cart');
        }
        else {
          uc_afffirm_transaction_updates($order_id, $response, $status = 'payment_received', $field = 'capture_trans_id');
          uc_order_comment_save($order_id, 0, t('Payment of @amount @currency submitted through Affirm.', array('@amount' => $order->order_total, '@currency' => 'USD')), 'order', $status = 'payment_received');
          uc_affirm_complete($order_id);
        }
      }
    }
    else {
      if ($response['status'] == 'authorized') {
        db_update('uc_orders')
            ->fields(array(
              'order_status' => 'order_under_review',
            ))
            ->condition('order_id', $order_id, '=')
            ->execute();
        db_update('uc_affirm')
            ->fields(array(
              'status' => 'order_under_review',
            ))
            ->condition('order_id', $order_id, '=')
            ->execute();
      }

      $order_id = intval($_SESSION['cart_order']);
      uc_affirm_complete($order_id);
    }
  }
  elseif ($response['code'] == 'auth-declined') {
    drupal_set_message(t('Affirm transaction authentication failed, Please try agin !!'), 'error');
    drupal_goto('cart');
  }
  else {
    get_affirm_status($response['code']);
  }
  return TRUE;
}

/**
 * Custom function to log transaction details.
 *
 * @param int $order_id
 *   Unique id for a order.
 * @param float $amount
 *   The order total amount.
 * @param string $payment_status
 *   The transaction status, that returned from affirm.
 * @param string $buyer_email
 *   The email address of the buyer.
 * @param int $authentication_trans_id
 *   The transaction id, that returned from affirm.
 * @param string $charge_id
 *   The payment transaction returned id.
 *
 * @return bool
 *   Returns TRUE if the transaction was successful. FALSE if it was not.
 */
function uc_affirm_transaction_save($order_id, $amount, $payment_status, $buyer_email, $authentication_trans_id, $charge_id = NULL) {
  db_insert('uc_affirm')
      ->fields(array(
        'order_id' => $order_id,
        'charge_id' => $charge_id,
        'amount' => $amount,
        'status' => $payment_status,
        'buyer_email' => $buyer_email,
        'received_time' => REQUEST_TIME,
        'authentication_trans_id' => $authentication_trans_id,
      ))
      ->execute();
  return TRUE;
}

/**
 * Function for getting charge id against the order id.
 *
 * @param int $order_id
 *   The unique id for an order.
 *
 * @return string
 *   Returns the charge id against the given order id.
 */
function _get_uc_affirm_charge_id($order_id) {
  $result = db_select('uc_affirm', 'n')
      ->fields('n', array('charge_id'))
      ->condition('order_id', $order_id, '=')
      ->execute()
      ->fetchAssoc();
  return $result['charge_id'];
}

/**
 * Form callback: allows the user to void/cancel a transaction.
 */
function uc_affirm_void_form($form, &$form_state, $order_id = NULL) {
  // Get charge_id form order id.
  $charge_id = _get_uc_affirm_charge_id($order_id);
  $form_state['order'] = $order_id;
  $form['markup'] = array(
    '#markup' => t('This action cannot be undone'),
  );

  $form = confirm_form($form, t('Are you sure that you want to void this transaction?'), 'admin/store/orders/view', '', t('Void'), t('Cancel'), 'confirm'
  );
  return $form;
}

/**
 * Submit handler: process a void/cancel request via Affirm.
 */
function uc_affirm_void_form_submit($form, &$form_state) {
  $order_id = $form_state['order'];
  // Get charge_id from the order id.
  $charge_id = _get_uc_affirm_charge_id($order_id);
  $response = uc_affirm_void($charge_id);

  // The call is valid and the payment gateway has been approved.
  if ($response) {
    if (isset($response['status_code'])) {
      drupal_set_message(t('Void failed'), 'error');
    }
    else {
      // Update status on uc_order table.
      uc_afffirm_transaction_updates($order_id, $response, $status = 'canceled', $field = 'void_trans_id');
      uc_order_comment_save($order_id, 0, t('The order canceled.'), 'order', $status = 'canceled');
      drupal_set_message(t('Transaction successfully voided.'));
    }
  }
  $form_state['redirect'] = 'admin/store/orders/' . $order_id;
}

/**
 * Form callback: allows the user to issue a refund on a prior transaction.
 */
function uc_affirm_refund_form($form, &$form_state, $order_id) {
  $order_id = arg(3);
  $order = uc_order_load($order_id, FALSE);

  // Getting the order date from time stamp fromat.
  $created_date = $order->created;
  $created_date = new DateTime("@$created_date");
  $created_date = $created_date->format('Y-m-d');

  // Getting the last date of refund.
  $date = date_create($created_date);
  date_add($date, date_interval_create_from_date_string('120 days'));
  $refundable_date = date_format($date, 'Y-m-d');

  // Checking the current date is greater than the refundable date.
  if (date("Y-m-d") > $refundable_date) {
    drupal_set_message(t('You cannot make a refund againt this order! Refund date expired!'));
    $form_state['redirect'] = 'admin/store/orders/' . $order_id;
    return;
  }

  $refund_data = db_query("SELECT * FROM {uc_affirm} WHERE order_id = :id", array(':id' => $order_id))->fetchObject();
  $default_amount = $order->order_total - $refund_data->refund_amount;
  if ($default_amount <= 0) {
    drupal_set_message(t('You cannot make a refund againt this order!'));
    $form_state['redirect'] = 'admin/store/orders/' . $order_id;
    return;
  }
  else {
    $form['amount'] = array(
      '#type' => 'textfield',
      '#title' => t('Refund amount'),
      '#description' => t('Enter the amount to be credited back to the original credit card(Doller).'),
      '#default_value' => $default_amount,
      '#size' => 16,
    );
    $form['order_id'] = array(
      '#type' => 'hidden',
      '#default_value' => $order_id,
      '#size' => 16,
    );

    $form = confirm_form($form, t('What amount do you want to credit?'), 'admin/commerce/orders/' . $order->order_id . '/payment', '', t('Credit'), t('Cancel'), 'confirm'
    );
    return $form;
  }
}

/**
 * Validate handler: check the refund amount before attempting refund request.
 */
function uc_affirm_refund_form_validate($form, &$form_state) {
  $refund_amount = $form_state['values']['amount'];
  $order_id = $form_state['values']['order_id'];
  $order = uc_order_load($order_id, FALSE);
  $refund_data = db_query("SELECT * FROM {uc_affirm} WHERE order_id = :id", array(':id' => $order_id))->fetchObject();
  // Ensure a positive numeric amount has been entered for refund.
  if (!is_numeric($refund_amount) || $refund_amount <= 0) {
    form_set_error('amount', t('You must specify a positive numeric amount to refund.'));
  }

  // Ensure the amount is less than or equal to the captured amount.
  if ($refund_data->refund_amount + $refund_amount > $order->order_total) {
    form_set_error('amount', t('You cannot refund more than you captured.'));
  }
}

/**
 * Submit handler: process a refund via Affirm.
 *
 * Sending the amounts in cent, not in dollers.
 * For that multiply the doller amount with 100.
 */
function uc_affirm_refund_form_submit($form, &$form_state) {
  $order_id = $form_state['values']['order_id'];
  $refund_amount = $form_state['values']['amount'];
  $order = uc_order_load($order_id, FALSE);
  $txn_type = UC_AFFIRM_REFUND;
  $charge_id = _get_uc_affirm_charge_id($order_id);
  $data = array(
    'amount' => number_format((float) $refund_amount, 2, '.', '') * 100,
  );
  $response = uc_affirm_api_request($txn_type, $order, $charge_id, $data);
  if ($response) {
    if (isset($response['status_code'])) {
      drupal_set_message(t('Refund failed'), 'error');
    }
    else {
      drupal_set_message(t('Refund for @amount issued successfully.', array('@amount' => $refund_amount, '@currency' => 'USD')));
      $refund_data = db_query("SELECT * FROM {uc_affirm} WHERE order_id = :id", array(':id' => $order_id))->fetchObject();
      // Caluculating the refund amount.
      $refund_amount = $refund_data->refund_amount + $refund_amount;
      // Update status on uc_order table.
      uc_afffirm_transaction_updates($order_id, $response, $status = 'chargeback', $field = 'refund_trans_id', $refund_amount);
      // Save the comment.
      uc_order_comment_save($order_id, 0, t('Refund of @amount @currency submitted through Affirm.', array('@amount' => $refund_amount, '@currency' => 'USD')), 'order', 'chargeback');
    }
  }
  $form_state['redirect'] = 'admin/store/orders/' . $order_id;
}

/**
 * Form callback: allows the user to capture a prior authorization.
 */
function uc_affirm_capture_form($form, &$form_state, $order_id) {
  $order_id = arg(3);
  $order = uc_order_load($order_id, FALSE);

  $form['amount'] = array(
    '#type' => 'markup',
    '#markup' => t('Capture amount: @amount', array('@amount' => uc_currency_format($order->order_total, $order->currency))),
  );
  $form['order_id'] = array(
    '#type' => 'hidden',
    '#default_value' => $order_id,
    '#size' => 16,
  );
  $form = confirm_form($form, t('Are you sure you want to capture this transaction?'), 'admin/store/orders/view', '', t('Capture Transaction'), t('Cancel'), 'confirm'
  );

  return $form;
}

/**
 * Submit handler: process a prior authorization capture via AIM.
 */
function uc_affirm_capture_form_submit($form, &$form_state) {
  $order_id = $form_state['values']['order_id'];
  $order = uc_order_load($order_id, FALSE);
  $txn_type = UC_AFFIRM_CAPTURE_ONLY;
  $charge_id = _get_uc_affirm_charge_id($order_id);
  $data = array(
    'order_id' => $order_id,
  );
  $response = uc_affirm_api_request($txn_type, $order, $charge_id, $data);
  if (empty($response)) {
    drupal_set_message(t('We could not complete your payment with Affirm. Please try again or contact us if the problem persists.'), 'error');
    return FALSE;
  }
  else {
    if (isset($response['status_code'])) {
      if ($response['status_code'] == 'capture-unauthorized') {
        drupal_set_message(t('Unauthorized capture request.'), 'error');
        $form_state['redirect'] = 'admin/store/orders/' . $order_id;
      }
      elseif ($response['status_code'] == 'capture-declined') {
        drupal_set_message(t('Affirm amount capture failed, Please try agin !!'), 'error');
        $form_state['redirect'] = 'admin/store/orders/' . $order_id;
      }
    }
    else {
      drupal_set_message(t('Captured successfully.'));
      uc_afffirm_transaction_updates($order_id, $response, $status = 'payment_received', $field = 'capture_trans_id');
      uc_order_comment_save($order_id, 0, t('Payment of @amount @currency submitted through Affirm.', array('@amount' => $order->order_total, '@currency' => 'USD')), 'order', $status = 'payment_received');
      $form_state['redirect'] = 'admin/store/orders/' . $order_id;
    }
  }
}

/**
 * Function to handle the updations in the order table and uc_affirm table.
 *
 * @param int $order_id
 *   Unique id for a order.
 * @param array $response
 *   The the response data from affirm.
 * @param string $status
 *   Status to be updated in the tables.
 * @param string $field
 *   The fields names for updating the transaction id's.
 * @param float $refund
 *   The refund amount that returned to the customer.
 *
 * @return bool
 *   Returns TRUE if the transaction was successful. FALSE if it was not.
 */
function uc_afffirm_transaction_updates($order_id, $response, $status, $field, $refund = NULL) {
  $order = uc_order_load($order_id, FALSE);
  db_update('uc_orders')
      ->fields(array(
        'order_status' => $status,
      ))
      ->condition('order_id', $order_id, '=')
      ->execute();

  $uc_affirm_update = db_update('uc_affirm');
  $uc_affirm_update->fields(array(
    'status' => $status,
    $field => $response['transaction_id'],
  ));
  if ($refund != NULL) {
    $uc_affirm_update->fields(array(
      'refund_amount' => $refund,
    ));
  }
  $uc_affirm_update->condition('order_id', $order_id, '=')
      ->execute();

  // Here you need to put in the routines for a successful
  // transaction such as sending an email to customer,
  // setting database status, informing logistics etc.
  $comment = t('Affrim @transaction_id id', array('@transaction_id' => $response['transaction_id']));
  if (($field == "capture_trans_id") || ($field == "refund_trans_id")) {
    if ($field == "refund_trans_id") {
      $order->order_total = '-' . $response['amount'];
    }
    uc_payment_enter($order_id, 'Affirm', $order->order_total, $order->uid, NULL, $comment);
  }
  uc_cart_complete_sale($order);
  return TRUE;
}

/**
 * Track each response code and show proper messages against them.
 */
function get_affirm_status($response_code = NULL) {
  drupal_set_message(t('Transaction failed, Please try agin !!'), 'error');
  drupal_goto('cart');
}
